/**
 *   Copyright (c) Rich Hickey. All rights reserved.
 *   The use and distribution terms for this software are covered by the
 *   Eclipse Public License 1.0 (http://opensource.org/licenses/eclipse-1.0.php)
 *   which can be found in the file epl-v10.html at the root of this distribution.
 *   By using this software in any fashion, you are agreeing to be bound by
 * 	 the terms of this license.
 *   You must not remove this notice, or any other, from this software.
 **/

/* rich Mar 3, 2008 */

package com.urbancode.commons.util.immutable;

import java.util.Collection;
import java.util.Iterator;
import java.util.Set;

abstract class AImmutableSet<T> implements IImmutableSet<T>, Set<T> {
    int _hash = -1;
    final IImmutableMap<T, T> impl;

    protected AImmutableSet(IImmutableMap<T, T> impl) {
        this.impl = impl;
    }

    @Override
    public String toString() {
        return Util.toString(this, iterator());
    }

    public boolean contains(Object key) {
        return impl.containsKey(key);
    }

    public T get(T key) {
        return impl.get(key);
    }

    public int size() {
        return impl.size();
    }

    @Override
    public boolean equals(Object obj) {
        if (!(obj instanceof Set<?>)) {
            return false;
        }
        Set<?> set = (Set<?>) obj;

        if (set.size() != size() || set.hashCode() != hashCode()) {
            return false;
        }

        for (Object e : set) {
            if (!contains(e)) {
                return false;
            }
        }

        return true;
    }

    @Override
    public int hashCode() {
        if (_hash == -1) {
            // int hash = count();
            int hash = 0;
            for (T e : this) {
                hash += Util.hash(e);
            }
            this._hash = hash;
        }
        return _hash;
    }

    public boolean add(Object o) {
        throw new UnsupportedOperationException();
    }

    public boolean remove(Object o) {
        throw new UnsupportedOperationException();
    }

    public boolean addAll(Collection<? extends T> c) {
        throw new UnsupportedOperationException();
    }

    public void clear() {
        throw new UnsupportedOperationException();
    }

    public boolean retainAll(Collection<?> c) {
        throw new UnsupportedOperationException();
    }

    public boolean removeAll(Collection<?> c) {
        throw new UnsupportedOperationException();
    }

    public boolean containsAll(Collection<?> c) {
        for (Object o : c) {
            if (!contains(o)) {
                return false;
            }
        }
        return true;
    }

    public <E> E[] toArray(E[] array) {
        return Util.toArray(size(), iterator(), array);
    }

    public Object[] toArray() {
        return Util.toArray(size(), iterator());
    }

    public boolean isEmpty() {
        return size() == 0;
    }

    public Iterator<T> iterator() {
        return impl.keySet().iterator();
    }
}
