/**
 *   Copyright (c) Rich Hickey. All rights reserved.
 *   The use and distribution terms for this software are covered by the
 *   Eclipse Public License 1.0 (http://opensource.org/licenses/eclipse-1.0.php)
 *   which can be found in the file epl-v10.html at the root of this distribution.
 *   By using this software in any fashion, you are agreeing to be bound by
 * 	 the terms of this license.
 *   You must not remove this notice, or any other, from this software.
 **/

/* rich Mar 3, 2008 */

package com.urbancode.commons.util.immutable;

import java.util.Collection;

public class ImmutableHashSet<T> extends AImmutableSet<T> {

    static private final ImmutableHashSet<?> EMPTY = new ImmutableHashSet<Object>(ImmutableHashMap.empty());

    @SuppressWarnings("unchecked")
    static public <T> ImmutableHashSet<T> empty() {
        return (ImmutableHashSet<T>) EMPTY;
    }

    public static <T> ImmutableHashSet<T> create(T... init) {
        ImmutableHashSet<T> ret = empty();
        for (T element : init) {
            ret = ret.with(element);
        }
        return ret;
    }

    public static <T> ImmutableHashSet<T> create(Collection<? extends T> init) {
        ImmutableHashSet<T> ret = empty();
        for (T key : init) {
            ret = ret.with(key);
        }
        return ret;
    }

    ImmutableHashSet(IImmutableMap<T, T> impl) {
        super(impl);
    }

    public ImmutableHashSet<T> without(Object key) {
        if (contains(key)) {
            return new ImmutableHashSet<T>(impl.without(key));
        }
        return this;
    }

    public ImmutableHashSet<T> with(T o) {
        if (contains(o)) {
            return this;
        }
        return new ImmutableHashSet<T>(impl.with(o, o));
    }

    public ImmutableHashSet<T> union(Collection<? extends T> coll) {
        ImmutableHashSet<T> result = this;
        for (T t : coll) {
            result = result.with(t);
        }
        return result;
    }

    public ImmutableHashSet<T> difference(Collection<?> coll) {
        ImmutableHashSet<T> result = this;
        for (Object t : coll) {
            result = result.without(t);
        }
        return result;
    }

    public ImmutableHashSet<T> intersection(Collection<?> coll) {
        ImmutableHashSet<T> result = this;
        for (T t : this) {
            if (!coll.contains(t)) {
                result = result.without(t);
            }
        }
        return result;
    }
}
