/**
 *   Copyright (c) Rich Hickey. All rights reserved.
 *   The use and distribution terms for this software are covered by the
 *   Eclipse Public License 1.0 (http://opensource.org/licenses/eclipse-1.0.php)
 *   which can be found in the file epl-v10.html at the root of this distribution.
 *   By using this software in any fashion, you are agreeing to be bound by
 * 	 the terms of this license.
 *   You must not remove this notice, or any other, from this software.
 **/

/* rich Apr 19, 2008 */

package com.urbancode.commons.util.immutable;

import java.lang.reflect.Array;
import java.util.Iterator;
import java.util.List;

final class Util {
    final static Object[] EMPTY_ARRAY = new Object[] {};

    static boolean equals(Object k1, Object k2) {
        if (k1 == k2) {
            return true;
        }
        return k1 != null && k1.equals(k2);
    }

    static public int hash(Object o) {
        if (o == null) {
            return 0;
        }
        return o.hashCode();
    }

    @SuppressWarnings("unchecked")
    static <A> A[] newArray(Class<A> componentType, int length) {
        return (A[]) Array.newInstance(componentType, length);
    }

    @SuppressWarnings("unchecked")
    static <A> Class<A> componentType(Object array) {
        return (Class<A>) array.getClass().getComponentType();
    }

    @SuppressWarnings("unchecked")
    static <T, E> E[] toArray(int size, Iterator<T> itr, E[] array) {
        if (array.length < size) {
            array = Util.newArray(Util.<E>componentType(array), size);
        }
        for (int i = 0; itr.hasNext(); i++) {
            array[i] = (E)itr.next();
        }
        if (array.length > size) {
            array[size] = null;
        }
        return array;
    }

    static <T> Object[] toArray(int size, Iterator<T> itr) {
        Object[] array = new Object[size];
        for (int i = 0; itr.hasNext(); i++) {
            array[i] = itr.next();
        }
        return array;
    }

    static <T> String toString(Object self, Iterator<T> i) {
        if (! i.hasNext()) {
            return "[]";
        }

        StringBuilder sb = new StringBuilder();
        sb.append('[');
        for (;;) {
            T e = i.next();
            sb.append(e == self ? "(this Collection)" : e);
            if (!i.hasNext()) {
                return sb.append(']').toString();
            }
            sb.append(", ");
        }
    }

    static <T> int hashCode(List<T> list) {
        int hash = 1;
        for (T t : list) {
            hash = 31 * hash + (t == null ? 0 : t.hashCode());
        }
        return hash;
    }

    static <T> boolean equals(List<T> self, Object object) {
        if (object == self) {
            return true;
        }
        if (!(object instanceof List<?>)) {
            return false;
        }

        Iterator<T> e1 = self.iterator();
        Iterator<?> e2 = ((List<?>) object).iterator();
        while (e1.hasNext() && e2.hasNext()) {
            T o1 = e1.next();
            Object o2 = e2.next();
            if (!(o1 == null ? o2 == null : o1.equals(o2))) {
                return false;
            }
        }
        return !(e1.hasNext() || e2.hasNext());
    }


    private Util() {
    }
}
